// src/common/crypto/xaes.ts
async function aesBlock(key, data, xor) {
  const block = await crypto.subtle.encrypt(
    { name: "AES-CBC", iv: xor },
    key,
    data
  );
  return new Uint8Array(block, 0, 16);
}
async function halfKey(index, key, iv, k1) {
  const m = new Uint8Array(16);
  m[1] = index;
  m[2] = 88;
  m.set(iv.subarray(0, 12), 4);
  return await aesBlock(key, m, k1);
}
async function deriveKeyNonce(key, iv) {
  if (key.algorithm.name !== "AES-CBC") {
    throw new Error("key must be for AES-CBC");
  }
  if (!key.usages.includes("encrypt")) {
    throw new Error("key must have 'encrypt' usage");
  }
  if (key.algorithm.length !== 256) {
    throw new Error("key must be 256 bits");
  }
  if (iv == null || iv.byteLength !== 24) {
    throw new Error("iv must be 24 bytes");
  }
  const ivBytes = ArrayBuffer.isView(iv) ? new Uint8Array(iv.buffer, iv.byteOffset, iv.byteLength) : new Uint8Array(iv);
  const k1 = await aesBlock(key, new Uint8Array(16), new Uint8Array(16));
  let msb = 0;
  for (let i = k1.length - 1; i >= 0; i--) {
    [msb, k1[i]] = [k1[i] >>> 7 & 255, (k1[i] << 1 | msb) & 255];
  }
  k1[k1.length - 1] ^= msb * 135 & 255;
  const kxBytes = new Uint8Array(32);
  kxBytes.set(await halfKey(1, key, ivBytes, k1), 0);
  kxBytes.set(await halfKey(2, key, ivBytes, k1), 16);
  const kx = await crypto.subtle.importKey(
    "raw",
    kxBytes,
    { name: "AES-GCM", length: 256 },
    false,
    [...key.usages]
  );
  return {
    key: kx,
    iv: ivBytes.subarray(12)
  };
}
async function encrypt(params, key, data) {
  const derived = await deriveKeyNonce(key, params.iv);
  return await crypto.subtle.encrypt(
    {
      name: "AES-GCM",
      iv: derived.iv,
      tagLength: 128,
      additionalData: params.additionalData ?? new Uint8Array()
    },
    derived.key,
    data
  );
}
async function decrypt(params, key, data) {
  const derived = await deriveKeyNonce(key, params.iv);
  return await crypto.subtle.decrypt(
    {
      name: "AES-GCM",
      iv: derived.iv,
      tagLength: 128,
      additionalData: params.additionalData ?? new Uint8Array()
    },
    derived.key,
    data
  );
}
async function generateKey(extractable) {
  return await crypto.subtle.generateKey(
    {
      name: "AES-CBC",
      // @ts-expect-error todo
      length: 256
    },
    extractable,
    ["encrypt", "decrypt"]
  );
}
async function importKey(format, keyData, extractable) {
  return await crypto.subtle.importKey(
    // @ts-expect-error-next-line
    format,
    keyData,
    { name: "AES-CBC", length: 256 },
    extractable,
    ["encrypt", "decrypt"]
  );
}
async function exportKey(format, key) {
  return await crypto.subtle.exportKey(format, key);
}

export {
  encrypt,
  decrypt,
  generateKey,
  importKey,
  exportKey
};
//# sourceMappingURL=chunk-BET4I7US.js.map